unit Unit2;

interface

uses
  Classes, Forms, SysUtils, Graphics;

type
  RGBEntry=Record
             R,G,B:Byte;
           End;
  MBitmap=Array [0..255, 0..255] of RGBEntry;
  WBitmap=Array [0..255, 0..255] of Word;
  TVoxel=Record
           X,Y,Z:Byte;
           Color:Word;
         End;
  TVoxCreator = class(TThread)
  private
    { Private-Deklarationen }
  protected
    procedure Execute; override;
  end;

type
  DSP=Array[0..2048*1536] of Word;

var
  Mxz,Mxzh,Myz,Myzr,Mxy,Mxyu:MBitmap;
  xz,xzh,yz,yzr,xy,xyu:WBitmap;
  SMatrix:Array [0..255,0..255,0..255] of Byte;
  CMatrix:Array [0..255,0..255,0..255] of Word;
  Anz:0..16777216;
  TDPic:TPicture;
  TDSpace:TBitmap;
  pl:^DSP;
  cnt:Longint;
  NameEntered:Boolean;
  VName:String;

implementation

uses Unit1;

{ Wichtig: Methoden und Eigenschaften eines Objekts in der VCL
           knnen nur in einem Methodenaufruf mit SYNCHRONIZE
           genutzt werden, z.B.

      Synchronize(UpdateCaption);

  und UpdateCaption knnte sein,

    procedure VoxCreator.UpdateCaption;
    begin
      Form1.Caption := 'Updated in einem Thread';
    end; }

{ VoxCreator }

procedure LoadBitmap(FileName:String; var Buffer:MBitmap);
var
  F:File;
  i, j:Integer;
  bt:MBitmap;
begin
  Assign(F,FileName);
  Reset(F,1);
  Seek(F,$36);
  BlockRead(F, bt, SizeOf(bt));
  for i:=0 to 255 do begin
    for j:=0 to 255 do begin
      // Microsoft Idiotic Format -> Normal Picture
      Buffer[j, 255-i]:=bt[i, j];
    end;
  end;
  Close(F);
end;

procedure SetStatus(p1,p2:Integer);
begin
  Form1.Label3.Caption:=IntToStr(p2)+'%';
  Form1.ProgressBar1.Position:=p2;
  Form1.Label4.Caption:=IntToStr(p1)+'%';
  Form1.ProgressBar2.Position:=p1;
end;

procedure CalcColors(var MB:Mbitmap;var WB:WBitmap);
var
  i, j:Integer;
  Cl:RGBEntry;
begin
  For i:=0 to 255 do begin
    For j:=0 to 255 do begin
      Cl:=MB[i,j];
      // 32->16Bit(565), BGR->RGB
      WB[i, j]:=2048*(Cl.B shr 3)+32*(Cl.G shr 2)+Cl.R shr 3;
    end;
  end;
end;

procedure TVoxCreator.Execute;
var
  i, j, k:Integer;
  Vox:TVoxel;
  VF:File;
begin
  Form1.ListBox1.Items.Add('VoxelCreator Initiated.');
  while not NameEntered do Application.ProcessMessages;
  Form1.ListBox1.Items.Add('Loading Bitmaps and Calculating Colors...');
  LoadBitmap('Objects\'+VName+'\xy.bmp', Mxy);
  CalcColors(Mxy,xy);
  SetStatus(16,1);
  LoadBitmap('Objects\'+VName+'\xyu.bmp', Mxyu);
  CalcColors(Mxyu,xyu);
  SetStatus(33,2);
  LoadBitmap('Objects\'+VName+'\xz.bmp', Mxz);
  CalcColors(Mxz,xz);
  SetStatus(50,3);
  LoadBitmap('Objects\'+VName+'\xzh.bmp', Mxzh);
  CalcColors(Mxzh,xzh);
  SetStatus(66,4);
  LoadBitmap('Objects\'+VName+'\yz.bmp', Myz);
  CalcColors(Myz,yz);
  SetStatus(83,5);
  LoadBitmap('Objects\'+VName+'\yzr.bmp', Myzr);
  CalcColors(Myzr,yzr);
  SetStatus(100,6);
  Form1.ListBox1.Items.Add('Creating Shape Matrix...');
  SetStatus(0,6);
  For i:=0 to 255 do begin // X
    For j:=0 to 255 do begin // Y
      For k:=0 to 255 do begin // Z
        if (xy[i, j]<>0)and(xz[i, k]<>0)and(yz[j, k]<>0) then begin
          SMatrix[i, j, k]:=1;
        end;
      end;
    end;
    SetStatus(i*100 div 255,6+i*25 div 255);
  end;
  Form1.ListBox1.Items.Add('Creating colored Matrix...');
  For i:=0 to 255 do begin // X
    For j:=0 to 255 do begin // Y
      For k:=0 to 255 do begin // Z
        if SMatrix[i, j, k]<>0 then begin
          if (i>=0)and(i<255)and(j>=0)and(j<255)and(k>=0)and(k<255) then begin
            if   (SMatrix[i+1, j+1, k+1]<>0)
              and(SMatrix[i+1, j+1, k]<>0)
              and(SMatrix[i+1, j+1, k-1]<>0)
              and(SMatrix[i+1, j, k+1]<>0)
              and(SMatrix[i+1, j, k]<>0)
              and(SMatrix[i+1, j, k-1]<>0)
              and(SMatrix[i+1, j-1, k+1]<>0)
              and(SMatrix[i+1, j-1, k]<>0)
              and(SMatrix[i+1, j-1, k-1]<>0)
              and(SMatrix[i, j+1, k+1]<>0)
              and(SMatrix[i, j+1, k]<>0)
              and(SMatrix[i, j+1, k-1]<>0)
              and(SMatrix[i, j, k+1]<>0)
              and(SMatrix[i, j, k-1]<>0)
              and(SMatrix[i, j-1, k+1]<>0)
              and(SMatrix[i, j-1, k]<>0)
              and(SMatrix[i, j-1, k-1]<>0)
              and(SMatrix[i-1, j+1, k+1]<>0)
              and(SMatrix[i-1, j+1, k]<>0)
              and(SMatrix[i-1, j+1, k-1]<>0)
              and(SMatrix[i-1, j, k+1]<>0)
              and(SMatrix[i-1, j, k]<>0)
              and(SMatrix[i-1, j, k-1]<>0)
              and(SMatrix[i-1, j-1, k+1]<>0)
              and(SMatrix[i-1, j-1, k]<>0)
              and(SMatrix[i-1, j-1, k-1]<>0)
            then continue;
          end;
          // Erstelle CMatrixPunkt
          if SMatrix[i, j, k-1]=0 then CMatrix[i, j, k]:=xy[i, j]
          else if SMatrix[i, j, k+1]=0 then CMatrix[i, j, k]:=xyu[i, j]
          else if SMatrix[i-1, j, k]=0 then CMatrix[i, j, k]:=yz[j, k]
          else if SMatrix[i+1, j, k]=0 then CMatrix[i, j, k]:=yzr[j, k]
          else if SMatrix[i, j-1, k]=0 then CMatrix[i, j, k]:=xz[i, k]
          else if SMatrix[i, j+1, k]=0 then CMatrix[i, j, k]:=xzh[i, k]
//          else SMatrix[i, j, k]:=xy[i, j];
        end;
      end;
    end;
    SetStatus(i*100 div 255,31+i*25 div 255);
  end;
  Form1.ListBox1.Items.Add('Creating Voxelfile...');
  Assign(VF,'Objects\'+VName+'\'+VName+'.vox');
  Rewrite(VF,1);
  Blockwrite(VF,Anz,SizeOf(Anz));
  For k:=0 to 255 do begin // Z
    For i:=0 to 255 do begin // X
      For j:=0 to 255 do begin // Y
        If CMatrix[i, j, k]<>0 then Begin
          Inc(Anz);
          Vox.X:=i;
          Vox.Y:=j;
          Vox.Z:=k;
          Vox.Color:=CMatrix[i, j, k];
          Blockwrite(VF,Vox,SizeOf(Vox));
        End;
      end;
    end;
    SetStatus(i*100 div 255,52+i*44 div 255);
  end;
  Seek(VF,0);
  Blockwrite(VF,Anz,SizeOf(Anz));
  Close(VF);
  SetStatus(100,100);
  Form1.ListBox1.Items.Add('Done.');
  TDSpace:=TBitmap.Create;
  TDSpace.Width:=Form1.Image1.Width;
  TDSpace.Height:=Form1.Image1.Height;
  TDSpace.PixelFormat:=pf16bit;
  TDPic:=TPicture.Create;
  TDPic.Assign(TDSpace);
  Form1.Image1.Picture:=TDPic;
  pl:=TDSpace.Scanline[0];
  for i:=0 to Form1.Image1.Width-1 do begin
    for j:=0 to Form1.Image1.Height-1 do begin
      pl^[i-j*Form1.Image1.Width]:=$0000;
    end;
  end;
  while not Terminated do begin
    Application.ProcessMessages;
    for i:=0 to 255 do begin
      for j:=0 to 255 do begin
        pl^[i-j*Form1.Image1.Width]:=CMatrix[i, j, cnt mod 256];
      end;
    end;
    Form1.Image1.Canvas.Draw(0, 0, TDSpace);
    Inc(cnt);
  end;
end;

end.
